@@ -1,5 +1,5 @@
 from .main import CycleController
-from .conn import Timing, PSCycler, LinacPSCycler, PSCyclerFBP
+from .conn import Timing, PSCycler, LinacPSCycler, PSCyclerFBP, FOFBPSCycler
 from .util import get_psnames, get_sections, get_trigger_by_psname, \
     TRIGGER_NAMES
 from .bo_cycle_data import \
@@ -7,3 +7,4 @@ from .bo_cycle_data import \
     DEFAULT_RAMP_TOTDURATION, BASE_RAMP_CURVE_ORIG, \
     bo_get_default_waveform, bo_generate_base_waveform
 from .li_cycle_data import li_get_default_waveform
+from .fc_cycle_data import fc_get_default_waveform
