@@ -5,14 +5,14 @@ import time as _time
 import logging as _log
 import threading as _thread
 from concurrent.futures import ThreadPoolExecutor
-from epics import PV as _PV
 
 from ..namesys import Filter as _Filter, SiriusPVName as _PVName
 from ..search import PSSearch as _PSSearch
+from ..epics import PV as _PV
 
 from .conn import Timing, PSCycler, PSCyclerFBP, LinacPSCycler, FOFBPSCycler
 from .bo_cycle_data import DEFAULT_RAMP_DURATION
-from .util import get_sections as _get_sections, Const as _Const, \
+from .util import Const as _Const, \
     get_trigger_by_psname as _get_trigger_by_psname
 
 TIMEOUT_SLEEP = 0.1
@@ -29,7 +29,7 @@ class CycleController:
         """Initialize."""
         # initialize auxiliar variables
         self._mode = None
-        self._sections = list()
+        self._include_sitrims = None
         self._not_ctrl_ti = None
         self._cycle_duration = 0
         self._aux_cyclers = dict()
@@ -96,8 +96,10 @@ class CycleController:
                     new_cyclers[name] = PSCycler(name, self._ramp_config)
         self._cyclers = new_cyclers
 
-        # define section
-        self._sections = _get_sections(self._cyclers.keys())
+        # define if we will perform default SI cycle procedure
+        include_sifam = bool(_Filter.process_filters(
+            self._cyclers.keys(), {'sec': 'SI', 'sub': 'Fam', 'dis': 'PS'}))
+        self._include_sitrims = include_sifam and not self._isadv
 
         # define not_ctrl_ti variable
         ps_no_ti = _PSSearch.get_psnames({'sec': 'LI', 'dis': 'PS'})
@@ -108,11 +110,23 @@ class CycleController:
         # define triggers
         self._triggers = _get_trigger_by_psname(self._cyclers.keys())
 
-        if 'SI' in self._sections and not self._isadv:
+        if self._include_sitrims:
+            # independent fbp magnets
+            qs_c2 = _PSSearch.get_psnames(
+                {'sec': 'SI', 'sub': '[0-2][0-9]C2', 'dis': 'PS',
+                 'dev': 'QS'})
+            cv2_c2 = _PSSearch.get_psnames(
+                {'sec': 'SI', 'sub': '[0-2][0-9]C2', 'dis': 'PS',
+                 'dev': 'CV', 'idx': '2'})
+            chv_id = _PSSearch.get_psnames(
+                {'sec': 'SI', 'sub': '[0-2][0-9]S(A|B|P)', 'dis': 'PS',
+                 'dev': 'C(H|V)'})
+            indfbp = qs_c2 + cv2_c2 + chv_id
             # trims psnames
-            self.trimnames = _PSSearch.get_psnames(
-                {'sec': 'SI', 'sub': '[0-2][0-9](M|C).*', 'dis': 'PS',
-                 'dev': '(CH|CV|QS|QD.*|QF.*|Q[1-4])'})
+            trimnames = set(_PSSearch.get_psnames(
+                {'sec': 'SI', 'sub': '[0-2][0-9](M|C|S).*', 'dis': 'PS',
+                 'dev': '(CH|CV|QS|QD.*|QF.*|Q[1-4])'}))
+            self.trimnames = list(trimnames - set(indfbp))
 
             # trims triggers
             self._si_aux_triggers = [
@@ -121,15 +135,10 @@ class CycleController:
             self._triggers.update(self._si_aux_triggers)
 
             # move CV-2 and QS of C2 to trims group, if they are in cyclers
-            qs_c2 = _PSSearch.get_psnames(
-                {'sec': 'SI', 'sub': '[0-2][0-9]C2', 'dis': 'PS',
-                 'dev': 'QS'})
-            cv2_c2 = _PSSearch.get_psnames(
-                {'sec': 'SI', 'sub': '[0-2][0-9]C2', 'dis': 'PS',
-                 'dev': 'CV', 'idx': '2'})
-            for psn in qs_c2 + cv2_c2:
+            for psn in indfbp:
                 if psn in self._cyclers.keys():
                     self._aux_cyclers[psn] = self._cyclers.pop(psn)
+                    self.trimnames.append(psn)
         else:
             self.trimnames = list()
 
@@ -206,11 +215,6 @@ class CycleController:
         """Mode."""
         return self._mode
 
-    @property
-    def sections(self):
-        """Mode."""
-        return self._sections
-
     @property
     def save_timing_size(self):
         """Save timing initial state task size."""
@@ -225,7 +229,7 @@ class CycleController:
     def prepare_ps_sofbmode_size(self):
         """Prepare PS SOFBMode task size."""
         prepare_ps_size = 2*(len(self.psnames)+1)
-        if 'SI' in self._sections and not self._isadv:
+        if self._include_sitrims:
             prepare_ps_size += 2*len(self.trimnames)
         return prepare_ps_size
 
@@ -233,7 +237,7 @@ class CycleController:
     def prepare_ps_opmode_slowref_size(self):
         """Prepare PS OpMode SlowRef task size."""
         prepare_ps_size = 2*(len(self.psnames)+1)
-        if 'SI' in self._sections and not self._isadv:
+        if self._include_sitrims:
             prepare_ps_size += 3*len(self.trimnames)
         return prepare_ps_size
 
@@ -241,7 +245,7 @@ class CycleController:
     def prepare_ps_current_zero_size(self):
         """Prepare PS current zero task size."""
         prepare_ps_size = 2*(len(self.psnames)+1)
-        if 'SI' in self._sections and not self._isadv:
+        if self._include_sitrims:
             prepare_ps_size += 3*len(self.trimnames)
         return prepare_ps_size
 
@@ -249,7 +253,7 @@ class CycleController:
     def prepare_ps_params_size(self):
         """Prepare PS parameters task size."""
         prepare_ps_size = 2*(len(self.psnames)+1)
-        if 'SI' in self._sections and not self._isadv:
+        if self._include_sitrims:
             prepare_ps_size += 3*len(self.trimnames)
         return prepare_ps_size
 
@@ -303,7 +307,7 @@ class CycleController:
     def prepare_ps_sofbmode_max_duration(self):
         """Prepare PS SOFBMode task maximum duration."""
         prepare_ps_max_duration = 5 + TIMEOUT_CHECK
-        if 'SI' in self._sections and not self._isadv:
+        if self._include_sitrims:
             prepare_ps_max_duration += TIMEOUT_CHECK
         return prepare_ps_max_duration
 
@@ -311,7 +315,7 @@ class CycleController:
     def prepare_ps_opmode_slowref_max_duration(self):
         """Prepare PS OpMode SlowRef task maximum duration."""
         prepare_ps_max_duration = 10 + TIMEOUT_CHECK
-        if 'SI' in self._sections and not self._isadv:
+        if self._include_sitrims:
             prepare_ps_max_duration += 3*TIMEOUT_CHECK
         return prepare_ps_max_duration
 
@@ -319,7 +323,7 @@ class CycleController:
     def prepare_ps_current_zero_max_duration(self):
         """Prepare PS current zero task maximum duration."""
         prepare_ps_max_duration = 10 + TIMEOUT_CHECK
-        if 'SI' in self._sections and not self._isadv:
+        if self._include_sitrims:
             prepare_ps_max_duration += 3*TIMEOUT_CHECK
         return prepare_ps_max_duration
 
@@ -327,7 +331,7 @@ class CycleController:
     def prepare_ps_params_max_duration(self):
         """Prepare PS parameters task maximum duration."""
         prepare_ps_max_duration = 10 + TIMEOUT_CHECK
-        if 'SI' in self._sections and not self._isadv:
+        if self._include_sitrims:
             prepare_ps_max_duration += 3*TIMEOUT_CHECK
         return prepare_ps_max_duration
 
@@ -335,7 +339,7 @@ class CycleController:
     def prepare_ps_opmode_cycle_max_duration(self):
         """Prepare PS task maximum duration."""
         prepare_ps_max_duration = 10 + TIMEOUT_CHECK
-        if 'SI' in self._sections and not self._isadv:
+        if self._include_sitrims:
             prepare_ps_max_duration += 3*TIMEOUT_CHECK
         return prepare_ps_max_duration
 
@@ -504,7 +508,6 @@ class CycleController:
         """Initialize trims cycling process."""
         # initialize dict to check which trim is cycling
         self._is_cycling_dict = {ps: True for ps in trims}
-
         # trigger
         self.trigger_timing()
 
@@ -666,7 +669,7 @@ class CycleController:
         psnames = {
             p for p in psnames if _PSSearch.conv_psname_2_psmodel(p) == 'FBP'}
         if not psnames:
-            return True
+            return
 
         self._update_log('Turning off power supplies SOFBMode...')
         for idx, psname in enumerate(psnames):
@@ -719,7 +722,7 @@ class CycleController:
         """Set power supplies OpMode to SlowRef."""
         psnames = {p for p in psnames if 'LI' not in p}
         if not psnames:
-            return True
+            return
 
         self._update_log('Setting power supplies to SlowRef...')
         with ThreadPoolExecutor(max_workers=100) as executor:
@@ -765,7 +768,8 @@ class CycleController:
         for psname, sts in self._checks_result.items():
             if sts:
                 continue
-            self._update_log(psname+' is not in SlowRef.', error=True)
+            opmdes = 'manual' if 'FC' in psname else 'SlowRef'
+            self._update_log(psname+' is not in '+opmdes+'.', error=True)
             status &= False
         return status
 
@@ -813,6 +817,23 @@ class CycleController:
             status &= False
         return status
 
+    def clear_pwrsupplies_fofbacc(self, psnames):
+        """Send clear accumulator command to FOFB power supplies."""
+        psnames = {
+            p for p in psnames
+            if _PSSearch.conv_psname_2_psmodel(p) == 'FOFB_PS'
+        }
+        if not psnames:
+            return
+
+        for idx, psname in enumerate(psnames):
+            cycler = self._get_cycler(psname)
+            cycler.clear_fofbacc()
+            if idx % 5 == 4 or idx == len(psnames)-1:
+                self._update_log(
+                    'Sent clear FOFBAcc command to {0}/{1}'.format(
+                        str(idx+1), str(len(psnames))))
+
     # --- main commands ---
 
     def save_timing_initial_state(self):
@@ -832,7 +853,7 @@ class CycleController:
         """Prepare SOFBMode."""
         psnames = self.psnames
         timeout = TIMEOUT_CHECK
-        if 'SI' in self._sections and not self._isadv:
+        if self._include_sitrims:
             self.create_trims_cyclers()
             psnames.extend(self.trimnames)
             timeout += TIMEOUT_CHECK
@@ -848,7 +869,7 @@ class CycleController:
         """Prepare OpMode to slowref."""
         psnames = self.psnames
         timeout = TIMEOUT_CHECK
-        if 'SI' in self._sections and not self._isadv:
+        if self._include_sitrims:
             self.create_trims_cyclers()
             psnames.extend(self.trimnames)
             timeout += 3*TIMEOUT_CHECK
@@ -856,7 +877,8 @@ class CycleController:
         self.set_pwrsupplies_slowref(psnames)
         if not self.check_pwrsupplies_slowref(psnames, timeout):
             self._update_log(
-                'There are power supplies not in OpMode SlowRef.', error=True)
+                'There are power supplies not in '
+                'the correct OpMode.', error=True)
             return
         self._update_log('Power supplies OpMode preparation finished!')
 
@@ -864,7 +886,7 @@ class CycleController:
         """Prepare current to cycle."""
         psnames = self.psnames
         timeout = TIMEOUT_CHECK
-        if 'SI' in self._sections and not self._isadv:
+        if self._include_sitrims:
             self.create_trims_cyclers()
             psnames.extend(self.trimnames)
             timeout += 3*TIMEOUT_CHECK
@@ -880,7 +902,7 @@ class CycleController:
         """Prepare parameters to cycle."""
         psnames = self.psnames
         timeout = TIMEOUT_CHECK
-        if 'SI' in self._sections and not self._isadv:
+        if self._include_sitrims:
             self.create_trims_cyclers()
             psnames.extend(self.trimnames)
             timeout += 3*TIMEOUT_CHECK
@@ -905,7 +927,7 @@ class CycleController:
 
     def cycle_trims(self):
         """Cycle all trims."""
-        if 'SI' not in self._sections or self._isadv:
+        if not self._include_sitrims:
             return
 
         if not self.check_timing():
@@ -915,18 +937,18 @@ class CycleController:
 
         self._update_log('Preparing to cycle CHs, QSs and QTrims...')
         trims = _PSSearch.get_psnames({
-            'sec': 'SI', 'sub': '[0-2][0-9](M|C).*', 'dis': 'PS',
+            'sec': 'SI', 'sub': '[0-2][0-9](M|C|S).*', 'dis': 'PS',
             'dev': '(CH|QS|QD.*|QF.*|Q[1-4])'})
-        if not self.cycle_trims_subset(trims, timeout=50):
+        if not self.cycle_trims_subset(trims, timeout=4*TIMEOUT_CHECK):
             self._update_log(
                 'There was problems in trims cycling. Stoping.', error=True)
             return
 
         self._update_log('Preparing to cycle CVs...')
         trims = _PSSearch.get_psnames({
-            'sec': 'SI', 'sub': '[0-2][0-9](M|C).*', 'dis': 'PS',
+            'sec': 'SI', 'sub': '[0-2][0-9](M|C|S).*', 'dis': 'PS',
             'dev': 'CV'})
-        if not self.cycle_trims_subset(trims, timeout=50):
+        if not self.cycle_trims_subset(trims, timeout=4*TIMEOUT_CHECK):
             self._update_log(
                 'There was problems in trims cycling. Stoping.', error=True)
             return
@@ -953,7 +975,7 @@ class CycleController:
             return
 
         triggers = self._triggers.copy()
-        if 'SI' in self._sections and not self._isadv:
+        if self._include_sitrims:
             triggers.difference_update(self._si_aux_triggers)
         if not self.set_triggers_state(triggers, 'enbl'):
             return
@@ -965,6 +987,7 @@ class CycleController:
         self.set_pwrsupplies_slowref(self.psnames)
         if not self.check_pwrsupplies_slowref(self.psnames):
             return False
+        self.clear_pwrsupplies_fofbacc(self.psnames)
 
         # Indicate cycle end
         self._update_log('Cycle finished!')
@@ -979,11 +1002,8 @@ class CycleController:
 
     def _filter_psnames(self, psnames2filt, filt):
         if psnames2filt:
-            psnames = _Filter.process_filters(
-                psnames2filt, filters=filt)
-        else:
-            psnames = _PSSearch.get_psnames(filt)
-        return psnames
+            return _Filter.process_filters(psnames2filt, filters=filt)
+        return _PSSearch.get_psnames(filt)
 
     def _get_cycler(self, psname):
         if psname in self._cyclers:
