@@ -2,34 +2,37 @@
 
 from .bbb import BunchbyBunch
 from .blscreen import BeamlineScreen
-from .bpm import BPM, FamBPMs
+from .bpm import BPM, FamBPMs, BPMLogicalTrigger
 from .currinfo import CurrInfoTranspEff, CurrInfoLinear, \
     CurrInfoBO, CurrInfoSI, CurrInfoAS
 from .dcct import DCCT
-from .device import Device, DeviceApp, Devices
+from .device import Device, DeviceApp, Devices, DeviceNC
 from .egun import EGBias, EGFilament, EGHVPS, EGTriggerPS, EGPulsePS, EGun
 from .energy import Energy
+from .fofb import FOFBCtrlDCC, BPMDCC, FOFBCtrlRef, FamFOFBControllers, \
+    FamFastCorrs, HLFOFB
 from .ict import ICT, TranspEff
 from .ids import IDCorrectors, APU, APUFeedForward
 from .injsys import ASPUStandbyHandler, BOPSRampStandbyHandler, \
     BORFRampStandbyHandler, InjBOStandbyHandler, InjSysStandbyHandler, \
-    LILLRFStandbyHandler
+    LILLRFStandbyHandler, InjSysPUModeHandler
 from .lillrf import LILLRF, DevLILLRF
 from .machshift import MachShift
 from .modltr import LIModltr
 from .orbit_interlock import BPMOrbitIntlk, BaseOrbitIntlk, OrbitInterlock
+from .posang import PosAng
 from .psconv import PSProperty, StrengthConv
 from .pssofb import PSCorrSOFB, PSApplySOFB
-from .pwrsupply import PowerSupply, PowerSupplyPU
+from .pwrsupply import PowerSupply, PowerSupplyPU, PowerSupplyFC
 from .rf import RFGen, ASLLRF, BORFCavMonitor, SIRFCavMonitor, RFCav, \
     RFKillBeam
 from .screen import Screen
 from .sofb import SOFB
 from .syncd import DevicesSync
-from .timing import EVG, Event, Trigger
+from .timing import EVG, Event, Trigger, HLTiming
 from .tune import TuneFrac, TuneProc, Tune, TuneCorr
 
 del device, bpm, dcct, egun, ict, lillrf, modltr,
-del pwrsupply, psconv, pssofb, rf, injsys
+del pwrsupply, posang, psconv, pssofb, rf, injsys
 del screen, tune, sofb, timing, syncd, energy
 del ids, currinfo, blscreen, bbb, machshift
