@@ -1,5 +1,7 @@
 """Power Supply Devices."""
 
+import numpy as _np
+
 from .. import util as _util
 
 from ..namesys import SiriusPVName as _SiriusPVName
@@ -23,7 +25,7 @@ class _PSDev(_Device):
         'Current-SP', 'Current-RB', 'Current-Mon',
     )
     _properties_magps = (
-        'Current-SP', 'Current-RB', 'Current-Mon',  'CurrentRef-Mon',
+        'Current-SP', 'Current-RB', 'Current-Mon', 'CurrentRef-Mon',
         'OpMode-Sel', 'OpMode-Sts',
         'WfmUpdateAuto-Sel', 'WfmUpdateAuto-Sts',
         'CycleType-Sel', 'CycleType-Sts',
@@ -34,9 +36,29 @@ class _PSDev(_Device):
         'CycleAuxParam-SP', 'CycleAuxParam-RB',
         'CycleEnbl-Mon',
     )
+    _properties_fc = (
+        'AlarmsAmp-Mon', 'OpMode-Sel', 'OpMode-Sts',
+        'CurrLoopKp-RB', 'CurrLoopKp-SP', 'CurrLoopTi-RB', 'CurrLoopTi-SP',
+        'CurrLoopMode-Sts', 'CurrLoopMode-Sel',
+        'CurrGain-RB', 'CurrGain-SP', 'CurrOffset-RB', 'CurrOffset-SP',
+        'Current-RB', 'Current-SP', 'Current-Mon', 'CurrentRef-Mon',
+        'TestLimA-RB', 'TestLimA-SP', 'TestLimB-RB', 'TestLimB-SP',
+        'TestWavePeriod-RB', 'TestWavePeriod-SP',
+        'Voltage-RB', 'Voltage-SP', 'Voltage-Mon',
+        'VoltGain-RB', 'VoltGain-SP', 'VoltOffset-RB', 'VoltOffset-SP',
+        'InvRespMatRow-SP', 'InvRespMatRow-RB',
+        'FOFBAccGain-SP', 'FOFBAccGain-RB',
+        'FOFBAccFreeze-Sel', 'FOFBAccFreeze-Sts',
+        'FOFBAccClear-Cmd',
+    )
     _properties_pulsed = (
         'Voltage-SP', 'Voltage-RB', 'Voltage-Mon',
         'Pulse-Sel', 'Pulse-Sts')
+    _properties_pulsed_sept = (
+        'Intlk1-Mon', 'Intlk2-Mon', 'Intlk3-Mon', 'Intlk4-Mon',
+        'Intlk5-Mon', 'Intlk6-Mon', 'Intlk7-Mon',
+    )
+    _properties_pulsed_kckr = _properties_pulsed_sept + ('Intlk8-Mon', )
 
     def __init__(self, devname):
         """."""
@@ -49,8 +71,8 @@ class _PSDev(_Device):
         # power supply type and magnetic function
         (self._pstype, self._psmodel, self._magfunc,
          self._strength_propty, self._strength_units,
-         self._is_linac, self._is_pulsed, self._is_magps) = \
-            _PSDev.get_device_type(devname)
+         self._is_linac, self._is_pulsed, self._is_fc,
+         self._is_magps) = _PSDev.get_device_type(devname)
 
         # set attributes
         (self._strength_sp_propty,
@@ -83,9 +105,14 @@ class _PSDev(_Device):
 
     @property
     def is_pulsed(self):
-        """Return True if device is a pulsed magnet powet supply."""
+        """Return True if device is a pulsed magnet power supply."""
         return self._is_pulsed
 
+    @property
+    def is_fc(self):
+        """Return True if device is a Sirius fast corrector power supply"""
+        return self._is_fc
+
     @property
     def is_magps(self):
         """Return True if device is a Sirius magnet power supply."""
@@ -146,10 +173,11 @@ class _PSDev(_Device):
         strength_units = _util.get_strength_units(magfunc, pstype)
         is_linac = devname.sec.endswith('LI')
         is_pulsed = devname.dis == 'PU'
-        is_magps = not is_linac and not is_pulsed
+        is_fc = devname.dev == 'FCH' or devname.dev == 'FCV'
+        is_magps = not is_linac and not is_pulsed and not is_fc
         return (pstype, psmodel, magfunc,
                 strength_propty, strength_units,
-                is_linac, is_pulsed, is_magps)
+                is_linac, is_pulsed, is_fc, is_magps)
 
     # --- private methods ---
 
@@ -158,11 +186,16 @@ class _PSDev(_Device):
         properties = _PSDev._properties_common
         if self._is_linac:
             properties += _PSDev._properties_linac
-        else:
-            if self._is_pulsed:
-                properties += _PSDev._properties_pulsed
+        elif self._is_pulsed:
+            properties += _PSDev._properties_pulsed
+            if self._psmodel == 'FP_KCKR':
+                properties += _PSDev._properties_pulsed_kckr
             else:
-                properties += _PSDev._properties_magps
+                properties += _PSDev._properties_pulsed_sept
+        elif self._is_fc:
+            properties += _PSDev._properties_fc
+        else:
+            properties += _PSDev._properties_magps
 
         # strength properties
         strength_sp_propty = self._strength_propty + '-SP'
@@ -547,6 +580,15 @@ class PowerSupplyPU(_PSDev):
         return set(list(super().pvnames) +
                    list(self._dev_timing.pvnames))
 
+    @property
+    def interlock_ok(self):
+        """Return whether all interlocks are in Ok state."""
+        intlks = [p for p in self._properties if 'Intlk' in p]
+        is_ok = True
+        for ilk in intlks:
+            is_ok &= self[ilk] == 1
+        return is_ok
+
     @property
     def connected(self):
         """Return PVs connection status."""
@@ -598,3 +640,169 @@ class PowerSupplyPU(_PSDev):
         devname = self._devname.substitute(dis='TI')
         device = _Device(devname, PowerSupplyPU._properties_timing)
         return device
+
+
+class PowerSupplyFC(_PSDev):
+    """Fast Correctors Power Supply Device."""
+
+    OPMODE_SEL = _Const.OpModeFOFBSel
+    OPMODE_STS = _Const.OpModeFOFBSts
+
+    class DEVICES:
+        """Devices names."""
+
+    @property
+    def opmode(self):
+        """OpMode."""
+        return self['OpMode-Sts']
+
+    @opmode.setter
+    def opmode(self, value):
+        self._enum_setter('OpMode-Sel', value, self.OPMODE_SEL)
+
+    @property
+    def current(self):
+        """Current setpoint."""
+        return self['Current-RB']
+
+    @current.setter
+    def current(self, value):
+        self['Current-SP'] = value
+
+    @property
+    def current_mon(self):
+        """Implemented current."""
+        return self['Current-Mon']
+
+    @property
+    def currentref_mon(self):
+        """Current reference."""
+        return self['CurrentRef-Mon']
+
+    @property
+    def curr_gain(self):
+        """Current gain for A<->raw unit conversion."""
+        return self['CurrGain-RB']
+
+    @curr_gain.setter
+    def curr_gain(self, value):
+        self['CurrGain-SP'] = value
+
+    @property
+    def curr_offset(self):
+        """Current offset for A<->raw unit conversion."""
+        return self['CurrOffset-RB']
+
+    @curr_offset.setter
+    def curr_offset(self, value):
+        self['CurrOffset-SP'] = value
+
+    @property
+    def voltage(self):
+        """."""
+        return self['Voltage-RB']
+
+    @voltage.setter
+    def voltage(self, value):
+        """."""
+        self['Voltage-SP'] = value
+
+    @property
+    def voltage_mon(self):
+        """."""
+        return self['Voltage-Mon']
+
+    @property
+    def volt_gain(self):
+        """."""
+        return self['VoltGain-RB']
+
+    @volt_gain.setter
+    def volt_gain(self, value):
+        """."""
+        self['VoltGain-SP'] = value
+
+    @property
+    def volt_offset(self):
+        """."""
+        return self['VoltOffset-RB']
+
+    @volt_offset.setter
+    def volt_offset(self, value):
+        """."""
+        self['VoltOffset-SP'] = value
+
+    @property
+    def currloop_kp(self):
+        """Current control loop Kp parameter."""
+        return self['CurrLoopKp-RB']
+
+    @currloop_kp.setter
+    def currloop_kp(self, value):
+        self['CurrLoopKp-SP'] = value
+
+    @property
+    def currloop_ti(self):
+        """Current control loop Ti parameter."""
+        return self['CurrLoopTi-RB']
+
+    @currloop_ti.setter
+    def currloop_ti(self, value):
+        self['CurrLoopTi-SP'] = value
+
+    @property
+    def currloop_mode(self):
+        """Current control loop mode."""
+        return self['CurrLoopMode-Sts']
+
+    @currloop_mode.setter
+    def currloop_mode(self, value):
+        self['CurrLoopMode-Sel'] = value
+
+    @property
+    def alarms_amp(self):
+        """."""
+        return self['AlarmsAmp-Mon']
+
+    def cmd_opmode_manual(self, timeout=_PSDev._default_timeout):
+        """Set opmode to manual."""
+        return self._set_opmode(mode=self.OPMODE_SEL.manual, timeout=timeout)
+
+    def cmd_opmode_fofb(self, timeout=_PSDev._default_timeout):
+        """Set opmode to fofb."""
+        return self._set_opmode(mode=self.OPMODE_SEL.fofb, timeout=timeout)
+
+    def _set_opmode(self, mode, timeout):
+        self['OpMode-Sel'] = mode
+        return self._wait('OpMode-Sts', mode, timeout=timeout)
+
+    @property
+    def invrespmat_row(self):
+        """Correction coefficient value."""
+        return self['InvRespMatRow-RB']
+
+    @invrespmat_row.setter
+    def invrespmat_row(self, value):
+        self['InvRespMatRow-SP'] = _np.array(value, dtype=float)
+
+    @property
+    def fofbacc_gain(self):
+        """FOFB accumulator gain."""
+        return self['FOFBAccGain-RB']
+
+    @fofbacc_gain.setter
+    def fofbacc_gain(self, value):
+        self['FOFBAccGain-SP'] = value
+
+    @property
+    def fofbacc_freeze(self):
+        """FOFB accumulator freeze state."""
+        return self['FOFBAccFreeze-Sts']
+
+    @fofbacc_freeze.setter
+    def fofbacc_freeze(self, value):
+        self['FOFBAccFreeze-Sel'] = value
+
+    def cmd_fofbacc_clear(self):
+        """Command to clear FOFB accumulator."""
+        self['FOFBAccClear-Cmd'] = 1
