@@ -351,11 +351,11 @@ class PSSearch:
     def get_splims_unit(psmodel):
         """Return SP limits unit."""
         PSSearch._reload_pstype_2_splims_dict()
-        if psmodel in ('FBP', 'FBP_DCLink', 'FBP_FOFB',
+        if psmodel in ('FBP', 'FBP_DCLink',
                        'FAC_DCDC', 'FAC_2S_DCDC', 'FAC_2S_ACDC',
                        'FAC_2P4S_DCDC', 'FAC_2P4S_ACDC',
                        'FAP', 'FAP_2P2S', 'FAP_4P',
-                       'Commercial', 'LINAC_PS'):
+                       'LINAC_PS', 'FOFB_PS'):
             return PSSearch._splims_ps_unit
         elif psmodel in ('APU', ):
             return 'mm'
