@@ -92,8 +92,8 @@ class Corrector(_BaseTimingConfig):
         """."""
         val = _PSConst.PwrStateSel.On if boo else _PSConst.PwrStateSel.Off
         pv = self._config_pvs_sp['PwrState']
-        if pv.connected:
-            self._config_ok_vals['PwrState'] = val
+        self._config_ok_vals['PwrState'] = val
+        if self.put_enable and pv.connected:
             pv.put(val, wait=False)
 
     @property
@@ -227,7 +227,7 @@ class CHCV(Corrector):
         """."""
         pvobj = self._config_pvs_sp['OpMode']
         self._config_ok_vals['OpMode'] = val
-        if pvobj.connected:
+        if self.put_enable and pvobj.connected:
             pvobj.put(val, wait=False)
 
     @property
@@ -252,7 +252,7 @@ class CHCV(Corrector):
         """."""
         pvobj = self._config_pvs_sp['SOFBMode']
         self._config_ok_vals['SOFBMode'] = val
-        if pvobj.connected:
+        if self.put_enable and pvobj.connected:
             pvobj.put(val, wait=False)
 
     @property
@@ -274,6 +274,9 @@ class CHCV(Corrector):
         """Configure method."""
         if not self.connected:
             return False
+
+        if not self.put_enable:
+            return True
         val = self._config_ok_vals['SOFBMode']
         self.sofbmode = 0
         for k, pvo in self._config_pvs_sp.items():
@@ -325,7 +328,7 @@ class Septum(Corrector):
         """."""
         pv = self._config_pvs_sp['Pulse']
         self._config_ok_vals['Pulse'] = val
-        if pv.connected and pv.value != val:
+        if self.put_enable and pv.connected and pv.value != val:
             pv.put(val, wait=False)
 
     @property
@@ -418,7 +421,7 @@ class TimingConfig(_BaseTimingConfig):
             return
         self._config_ok_vals['Src'] = value
         pvobj = self._config_pvs_sp['Src']
-        if pvobj.connected:
+        if self.put_enable and pvobj.connected:
             pvobj.value = value
 
     @property
@@ -433,7 +436,7 @@ class TimingConfig(_BaseTimingConfig):
     def delayraw(self, value):
         """."""
         pvobj = self._config_pvs_sp['DelayRaw']
-        if pvobj.connected:
+        if self.put_enable and pvobj.connected:
             pvobj.value = int(value)
 
     @property
@@ -461,6 +464,7 @@ class EpicsCorrectors(BaseCorrectors):
     def __init__(self, acc, prefix='', callback=None, dipoleoff=False):
         """Initialize the instance."""
         super().__init__(acc, prefix=prefix, callback=callback)
+        self._sofb = None
         self._sync_kicks = False
         self._acq_rate = 2
 
@@ -489,6 +493,15 @@ class EpicsCorrectors(BaseCorrectors):
             1/self._acq_rate, self._update_corrs_strength, niter=0)
         self._corrs_thread.start()
 
+    @property
+    def sofb(self):
+        """."""
+        return self._sofb
+
+    @sofb.setter
+    def sofb(self, sofb):
+        self._sofb = sofb
+
     @property
     def corrs(self):
         """."""
@@ -588,7 +601,11 @@ class EpicsCorrectors(BaseCorrectors):
         self._func_ret = func_ret
 
         self._set_ref_kicks(values)
-        self._pssofb.bsmp_sofb_kick_set(self._ref_kicks[-1][:-1])
+        # NOTE: values will contain NaNs when corrector is not in the loop, as
+        # a mean to not communicate with it. Pass them to PSSOFB so it handles
+        # this appropriately (actually it is the PSBSMP class that will handle
+        # NaNs):
+        self._pssofb.bsmp_sofb_kick_set(values[:-1])
         if not _np.isnan(values[-1]):
             self.put_value_in_corr(self._corrs[-1], values[-1])
 
@@ -687,7 +704,7 @@ class EpicsCorrectors(BaseCorrectors):
             _log.error(_traceback.format_exc())
 
     def set_corrs_mode(self, value):
-        """Set mode of CHs and CVs method."""
+        """Set mode of CHs and CVs method. Only called when acc==SI."""
         if value not in self._csorb.CorrSync:
             return False
         self._sync_kicks = value
@@ -702,15 +719,16 @@ class EpicsCorrectors(BaseCorrectors):
             self.set_timing_delay(self._csorb.CORR_DEF_DELAY)
             self.timing.sync_type = self.timing.CLK
 
-        for corr in self._corrs:
-            if corr.connected:
-                corr.opmode = val
-            else:
+        mask = self._get_mask()
+        for i, corr in enumerate(self._corrs[:-1]):
+            if mask[i] and not corr.connected:
                 msg = 'ERR: Failed to configure '
                 msg += corr.name
                 self._update_log(msg)
                 _log.error(msg[5:])
                 return False
+            corr.put_enable = mask[i]
+            corr.opmode = val
 
         strsync = self._csorb.CorrSync._fields[self._sync_kicks]
         msg = 'Synchronization set to {0:s}'.format(strsync)
@@ -751,13 +769,15 @@ class EpicsCorrectors(BaseCorrectors):
         # and neither why the problem even occurs...
         _time.sleep(0.2)
 
-        for corr in self._corrs[:-1]:
-            if not corr.connected:
+        mask = self._get_mask()
+        for i, corr in enumerate(self._corrs[:-1]):
+            if mask[i] and not corr.connected:
                 msg = 'ERR: Failed to configure '
                 msg += corr.name
                 self._update_log(msg)
                 _log.error(msg[5:])
                 continue
+            corr.put_enable = mask[i]
             corr.sofbmode = val
 
         self.run_callbacks('CorrPSSOFBEnbl-Sts', val)
@@ -775,7 +795,8 @@ class EpicsCorrectors(BaseCorrectors):
 
     def configure_correctors(self, _):
         """Configure correctors method."""
-        for corr in self._corrs:
+        corrs = self._get_used_corrs(include_rf=True)
+        for corr in corrs:
             if not corr.connected:
                 msg = 'ERR: Failed to configure '
                 msg += corr.name
@@ -804,7 +825,8 @@ class EpicsCorrectors(BaseCorrectors):
         elif self.isring:
             status = 0b0011111
 
-        chcvs = self._corrs[:self._csorb.nr_chcv]
+        chcvs = self._get_used_corrs(include_rf=False)
+
         status = _util.update_bit(
             status, bit_pos=0,
             bit_val=not all(corr.connected for corr in chcvs))
@@ -853,16 +875,20 @@ class EpicsCorrectors(BaseCorrectors):
         return True
 
     def _compare_kicks_pssofb(self, values, fret):
+        mask = self._get_mask()
         rfc = self._corrs[-1]
         ref_vals = self._ref_kicks[0]
 
-        res_tim = _np.ones(ref_vals.size, dtype=bool)
-        res_tim[:-1] = fret != _ConstPSBSMP.ACK_DSP_TIMEOUT
-        res = fret != _ConstPSBSMP.ACK_OK
-        res &= res_tim[:-1]
-        if _np.any(res):
-            self._print_guilty(
-                ~res, mode='prob_code', fret=fret)
+        # NOTE: In case there is some problem in the return value, apart from
+        # DSP_TIMEOUT, which we consider not a fatal problem, return code
+        # error -2 so that the loop can be opened:
+        timeout = _np.zeros(ref_vals.size, dtype=bool)
+        timeout[:-1] = fret == _ConstPSBSMP.ACK_DSP_TIMEOUT
+        comm_prob = fret != _ConstPSBSMP.ACK_OK
+        comm_prob &= ~timeout[:-1]
+        comm_prob &= mask[:-1]  # If corrector is not in the loop, ignore it.
+        if _np.any(comm_prob):
+            self._print_guilty(~comm_prob, mode='prob_code', fret=fret)
             return -2
 
         curr_vals = _np.zeros(self._csorb.nr_corrs, dtype=float)
@@ -875,6 +901,7 @@ class EpicsCorrectors(BaseCorrectors):
         iszero = _compare_kicks(curr_vals, 0)
         iszero_ref = _compare_kicks(ref_vals, 0)
         prob = iszero & ~(iszero_ref)
+        prob &= mask  # If corrector is not in the loop, ignore it.
         # For debugging:
         # if _np.any(prob):
         #     self._print_guilty(
@@ -890,9 +917,15 @@ class EpicsCorrectors(BaseCorrectors):
                 ~prob, mode='prob_curr', currs=curr_vals, refs=ref_vals)
             return -2
 
-        equl = _compare_kicks(curr_vals, ref_vals)
-        equl |= ~res_tim
-        return _np.sum(~equl)
+        diff = _np.logical_not(_compare_kicks(curr_vals, ref_vals))
+
+        # NOTE: I don't remember why it only matters when differences are due
+        # to Timeout error:
+        diff &= timeout
+
+        # If corrector is not in the loop, ignore it:
+        diff &= mask
+        return _np.sum(diff)
 
     def _print_guilty(
             self, okg, mode='ready', fret=None, currs=None, refs=None):
@@ -911,3 +944,20 @@ class EpicsCorrectors(BaseCorrectors):
                 msg = msg_tmpl.format(mode, corr.name, *args)
                 self._update_log(msg)
                 _log.error(msg[5:])
+
+    def _get_mask(self):
+        if self.sofb is not None and self.sofb.matrix is not None:
+            mask = self.sofb.matrix.corrs_enbllist
+        else:
+            mask = _np.ones(len(self._corrs), dtype=bool)
+        return mask
+
+    def _get_used_corrs(self, include_rf=False):
+        corrs = []
+        nrc = None if include_rf else self._csorb.nr_chcv
+        mask = self._get_mask()[:nrc]
+        for i, corr in enumerate(self._corrs[:nrc]):
+            corr.put_enable = mask[i]
+            if mask[i]:
+                corrs.append(corr)
+        return corrs
