@@ -15,7 +15,7 @@ from ..search import HLTimeSearch as _HLTimeSearch
 class ETypes(_csdev.ETypes):
     """Local enumerate types."""
 
-    EVT_MODES = ('Disabled', 'Continuous', 'Injection', 'OneShot', 'External')
+    EVT_MODES = ('Disable', 'Continuous', 'Injection', 'OneShot', 'External')
     TRIG_SRC_LL = (
         'Dsbl', 'Trigger', 'Clock0', 'Clock1', 'Clock2',
         'Clock3', 'Clock4', 'Clock5', 'Clock6', 'Clock7')
@@ -66,10 +66,10 @@ class Const(_csdev.Const):
     __EvtHL2LLMap = None
     __EvtLL2HLMap = None
 
-    evt_ll_codes = list(range(64)) + [124]
+    evt_ll_codes = list(range(64)) + [117, 124]
     evt_ll_names = ['Evt{0:02d}'.format(i) for i in evt_ll_codes]
     EvtLL = _csdev.Const.register(
-                    'EventsLL', evt_ll_names, values=evt_ll_codes)
+        'EventsLL', evt_ll_names, values=evt_ll_codes)
     del evt_ll_codes, evt_ll_names  # cleanup class namespace
 
     ClkHL2LLMap = {
